# -*- coding: utf-8 -*-
# pylint: disable-msg = C0103, C0321, W0613, W0704
#
# Copyright 2005-2006 André Malo or his licensors, as applicable
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Test utilities
"""
__author__    = "André Malo"
__docformat__ = "epytext en"
__all__       = ['mockModule', 'unmockModule', 'MockingBase']

import unittest


def mockModule(name, mdict = None, ispkg = False):
    """ mocks a module """
    import sys, imp

    class Importer(object):
        def __init__(self, fullname):
            self.name = fullname
        def find_module(self, fullname, path = None):
            if fullname == self.name: return self
            return None
        def load_module(self, fullname):
            if sys.modules.has_key(fullname):
                raise RuntimeError("module already exists?!")
            mod = sys.modules[fullname] = imp.new_module(fullname)
            mod.__file__ = '<mocked module %s' % fullname
            mod.__loader__ = None
            if ispkg: mod.__path__ = []
            if mdict: mod.__dict__.update(mdict)
            return mod

    sys.meta_path.insert(0, Importer(name))


def unmockModule(fullname):
    """ unmocks a module """
    import sys

    # remove mock loader
    sys.meta_path = [importer for importer in sys.meta_path
        if not(hasattr(importer, 'name') and importer.name == fullname)
    ]

    # remove from global sys.modules dict
    try:
        del sys.modules[fullname]
    except KeyError:
        # ignore
        pass

    # drop it also from the parent module
    idx = fullname.rfind('.')
    if idx != -1:
        parent = fullname[:idx]
        submodule = fullname[idx + 1:]
        try:
            del sys.modules[parent].__dict__[submodule]
        except KeyError:
            # ok
            pass


def loadModule(name):
    """ load a module """
    mod = __import__(name)
    components = name.split('.')
    for comp in components[1:]:
        mod = getattr(mod, comp)

    return mod


class MockingBase(unittest.TestCase):
    """ Mocking base """

    def mockModule(self, name, mdict):
        """ Mock module <name> using module dict <mdict>"""
        unmockModule(name)
        mockModule(name, mdict)
        mod = loadModule(name)

        if 'mocked' not in mod.__file__:
            raise RuntimeError("'%s' is %r?!" % (name, mod))

        return mod


    def unmockModule(self, name):
        """ Unmock module <name> """
        unmockModule(name)
        try:
            mod = loadModule(name)
        except ImportError:
            # in case the real module doesn't exist
            return

        try:
            mfile = mod.__file__
        except AttributeError:
            # no __file__ attribute is a good sign as well
            pass
        else:
            if 'mocked' in mfile:
                raise RuntimeError("module '%s' was not unmocked" % name)

        unmockModule(name)


    def addMock(self, name, initial):
        """ mock one module """
        self._tomock.append((name, initial))
        self._mocked[name] = self.mockModule(name, initial)


    def setUp(self):
        """ mock modules """
        self._mocked = {}
        self._tomock = []
        for name, initial in self.getInitialMocks():
            self.addMock(name, initial)


    def tearDown(self):
        """ Unload optparse """
        self._mocked.clear()
        while self._tomock:
            name = self._tomock.pop()[0]
            self.unmockModule(name)


    def getInitialMocks(self):
        """ Returns the list initially mocked modules """
        return []
